@@ -3,6 +3,6 @@
 
 // where a reference must be put
 
-typedef enum {INLINE=1, POSTLINE, POSTDOC} RefPosition;
+typedef enum {NONE = 0, INLINE = 1, POSTLINE, POSTDOC} RefPosition;
 
 #endif // REFPOSITION_H
