@@ -49,8 +49,8 @@
      COLOR = 265,
      BG_COLOR = 266,
      STRINGDEF = 267,
-     BODY_BG_COLOR = 268,
-     BG_T = 269
+     BG_T = 268,
+     BODY_BG_COLOR = 269
    };
 #endif
 /* Tokens.  */
@@ -64,15 +64,15 @@
 #define COLOR 265
 #define BG_COLOR 266
 #define STRINGDEF 267
-#define BODY_BG_COLOR 268
-#define BG_T 269
+#define BG_T 268
+#define BODY_BG_COLOR 269
 
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 54 "./styleparser.yy"
+#line 59 "./styleparser.yy"
 {
   int tok ; /* command */
   const std::string * string ; /* string : id, ... */
